const React = require('react')
const PropTypes = require('prop-types')


const CONTEXT_PROP = '@@mithrandir/instance'

class MithrandirProvider extends React.Component {
  getChildContext() {
    const context = {}
    context[CONTEXT_PROP] = this.props.instance
    return context
  }

  render() {
    return this.props.children
  }
}

MithrandirProvider.childContextTypes = {}
MithrandirProvider.childContextTypes[CONTEXT_PROP] = PropTypes.object

MithrandirProvider.propTypes = {
  instance: PropTypes.object.isRequired,
}

/**
 * MithrandirI18n component to render values
 * @example
 * (
 *  <div>
 *    <MithrandirI18n t="common.greetings" params={{ name: "Sergey Sova" }} />
 *  </div>
 * )
 */
function MithrandirI18n(props, context) {
  if (!context[CONTEXT_PROP]) {
    throw new TypeError('You should wrap your app with MithrandirProvider')
  }
  const [cat, name] = props.t.split('.', 1)
  const translated = context[CONTEXT_PROP].t(cat, name, props.params, props.def)

  return React.createElement(props.tag, {}, translated)
}

MithrandirI18n.contextTypes = {}
MithrandirI18n.contextTypes[CONTEXT_PROP] = PropTypes.object.isRequired

MithrandirI18n.propTypes = {
  /**
   * Set tag to render element
   */
  tag: PropTypes.string,

  /**
   * Default value if translated not found
   */
  def: PropTypes.string,

  /**
   * Category and name from mithrandir translation app
   * Format: category.name
   * Can be: category.name.second.name
   */
  t: PropTypes.string.isRequired,

  /**
   * Passed object to translation
   */
  params: PropTypes.object,

  /**
   * Children should be omitted
   */
  children(props, propName) {
    if (props[propName]) {
      return new TypeError('MithrandirI18n not use children')
    }
    return undefined
  },
}

MithrandirI18n.defaultProps = {
  tag: 'div',
  def: '',
  params: {},
}

module.exports = {
  MithrandirProvider,
  MithrandirI18n,
}
